<?php
/**
 * Output CSS for custom fonts in the <head>.
 */
namespace EStar\Fonts;

class CSS {
	private $elements;
	private $fonts;

	public function __construct() {
		$this->elements = Fonts::get_elements();

		// Using PHP include instead of get_template_part to get array of fonts config.
		// phpcs:ignore WPThemeReview.CoreFunctionality.FileInclude.FileIncludeFound
		$this->fonts    = include __DIR__ . '/google-fonts.php';

		add_action( 'wp_head', [ $this, 'enqueue' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_admin' ] );
		add_action( 'wp_head', [ $this, 'output_frontend' ], 20 );
		add_action( 'admin_head', [ $this, 'output_admin' ] );
	}

	public function enqueue() {
		$fonts  = [];
		$subset = [];

		// Font family are set in the Global section.
		$elements = ['base', 'headings'];
		$elements = array_merge( $elements, array_keys( $this->elements ) );

		foreach ( $elements as $id ) {
			$family = get_theme_mod( "{$id}_font_family" );

			// Don't import system fonts.
			if ( ! $family || in_array( $family, ['serif', 'sans-serif'] ) ) {
				continue;
			}
			$font = $this->get_font_settings( $family );
			if ( ! $font ) {
				continue;
			}
			$fonts[ $family ]   = isset( $fonts[ $family ] ) ? $fonts[ $family ] : [];
			$fonts[ $family ][] = get_theme_mod( "{$id}_font_style" );

			if ( in_array( $id, ['base', 'body'] ) ) {
				$fonts[ $family ] = array_merge( $fonts[ $family ], ['regular', 'italic', '700', '700italic'] );
			}
			if ( in_array( $id, ['headings', 'heading_1', 'heading_2', 'heading_3', 'heading_4', 'heading_5', 'heading_6', 'site_title', 'post_title_singular', 'post_title_archive'], true ) ) {
				$fonts[ $family ][] = '700';
			}

			$subset = array_merge( $subset, explode( ',', get_theme_mod( "{$id}_font_subsets" ) ) );
		}
		if ( ! $fonts ) {
			return;
		}
		foreach ( $fonts as $family => $styles ) {
			$styles = implode( ',', array_unique( array_filter( $styles ) ) );
			$fonts[ $family ] = $styles ? "$family:$styles" : $family;
		}

		$fonts_url = add_query_arg(
			array_filter( [
				'family'  => implode( '|', $fonts ),
				'subset'  => implode( ',', array_unique( array_filter( $subset ) ) ),
				'display' => 'swap',
			] ),
			'https://fonts.googleapis.com/css'
		);
		$fonts_url = esc_url( $fonts_url );

		echo "\n<link rel='preconnect' href='https://fonts.gstatic.com' crossorigin />";
		echo "\n<link rel='preload' as='style' href='$fonts_url' />";
		echo "\n<link rel='stylesheet' href='$fonts_url' media='print' onload='this.media=\"all\"' />\n";
	}

	public function enqueue_admin() {
		if ( $this->is_admin_screen() ) {
			$this->enqueue();
		}
	}

	public function output_frontend() {
		$css = $this->get_css();

		// Only include custom colors in customizer or frontend.
		if ( ! is_customize_preview() && ! $css ) {
			return;
		}

		// CSS is generated by the theme and using wp_strip_all_tags is safe.
		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo "\n<style id='estar-fonts'>\n\t", wp_strip_all_tags( implode( "\n\t", $css ) ), "\n</style>\n";
	}

	public function output_admin() {
		$css = $this->get_css();
		if ( ! $this->is_admin_screen() || ! $css ) {
			return;
		}

		// CSS is generated by the theme and using wp_strip_all_tags is safe.
		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo "\n<style id='estar-fonts'>\n\t", wp_strip_all_tags( implode( "\n\t", $css ) ), "\n</style>\n";
	}

	private function is_admin_screen() {
        $screen = get_current_screen();
        return $screen && $screen->is_block_editor();
	}

	private function get_css() {
		$css = [];
		$elements = [
			'base' => ['selector' => 'body'],
			'headings' => ['selector' => 'h1,h2,h3,h4,h5,h6'],
		];
		$elements = array_merge( $elements, $this->elements );
		foreach ( $elements as $id => $element ) {
			$css[] = $this->get_element_css( $id, $element );
		}
		$css = array_filter( $css );
		return $css;
	}

	/**
	 * Get CSS for a single element.
	 *
	 * @param  string $id      Element ID.
	 * @param  array  $element Element parameter.
	 *
	 * @return string
	 */
	private function get_element_css( $id, $element ) {
		$rules      = [];
		$selector   = $element['selector'];
		if ( is_admin() ) {
			$selector = '.editor-styles-wrapper.editor-styles-wrapper ' . str_replace( 'body', '', $selector );
		}
		$properties = array(
			'font_family',
			'font_style',
			'font_size',
			'line_height',
			'letter_spacing',
			'text_transform',
		);
		foreach ( $properties as $property ) {
			$value = get_theme_mod( "{$id}_{$property}" );
			if ( ! $value ) {
				continue;
			}
			switch ( $property ) {
				case 'font_family':
					if ( 'sans-serif' === $value ) {
						$value = 'var(--font-sans)';
					} elseif ( 'serif' === $value ) {
						$value = 'var(--font-serif)';
					} elseif ( $font = $this->get_font_settings( $value ) ) {
						$value = sprintf( '"%s", %s', $value, $font['category'] );
					}
					break;
				case 'font_style':
					$font_weight = $value;
					$font_style  = 'normal';
					if ( false !== strpos( $value, 'italic' ) ) {
						$font_weight = str_replace( 'italic', '', $value );
						$font_style  = 'italic';
					}
					$rules[] = "font-weight: $font_weight";
					if ( $font_style ) {
						$rules[] = "font-style: $font_style";
					}
					continue 2;
				case 'font_size':
				case 'letter_spacing':
					$value .= get_theme_mod( "{$id}_{$property}_unit", 'px' );
					break;
				case 'line_height':
					$value .= get_theme_mod( "{$id}_{$property}_unit" );
					break;
			}
			$rules[] = sprintf( '%s: %s', str_replace( '_', '-', $property ), $value );
		}

		return $rules ? "$selector { " . implode( '; ', $rules ) . ' }' : '';
	}

	private function get_font_settings( $family ) {
		foreach ( $this->fonts as $font ) {
			if ( $font['family'] === $family ) {
				return $font;
			}
		}
		return null;
	}
}
